@@ -4,17 +4,15 @@ use 5.006;
 use strict;
 use warnings;
 
+our $VERSION = '0.08';
+
 =head1 NAME
 
 Crypt::LE - Let's Encrypt API interfacing module.
 
 =head1 VERSION
 
-Version 0.07
-
-=cut
-
-our $VERSION = '0.07';
+Version 0.08
 
 =head1 SYNOPSIS
 
@@ -45,18 +43,32 @@ our $VERSION = '0.07';
 =head1 DESCRIPTION
 
 Crypt::LE provides the functionality necessary to use Let's Encrypt API and generate free SSL certificates for your domains. It can also
-be used to generate RSA keys and Certificate Signing Requests.
+be used to generate RSA keys and Certificate Signing Requests or to revoke previously issued certificates.
 
 Crypt::LE can be easily extended with custom plugins to handle Let's Encrypt challenges. See L<Crypt::LE::Challenge::Simple> module
-as an example of such plugin.
+for an example of a challenge-handling plugin.
 
-Crypt::LE is shipped with a self-sufficient client for obtaining SSL certificates - I<le.pl>.
+Crypt::LE is shipped with a self-sufficient client for obtaining SSL certificates - le.pl. Usage example:
 
-Usage example: B<le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing>
+ B<le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing>
 
 That will generate an account key and a CSR (plus key) if they are missing. If any of those files exist, they will just be loaded, so it is safe to re-run
-the client.
+the client. Run le.pl without any parameters or with C<--help> to see more details and usage examples.
 
+The provided client also supports challenge-handling plugins, such as L<Crypt::LE::Challenge::Simple>. In addition, it supports completion-handling
+plugins, such as L<Crypt::LE::Complete::Simple>. You can easily handle challenges and trigger specific actions when your certificate gets issued
+by using those modules as templates, without modifying the client code. You can also pass custom parameters to your modules from le.pl command line:
+
+ le.pl ... --handle-with Crypt::LE::Challenge::Simple --handle-params '{"key1": 1, "key2": "one"}'
+ 
+ le.pl ... --complete-with Crypt::LE::Complete::Simple --complete-params '{"key1": 1, "key2": "one"}'
+ 
+The parameters don't have to be put directly in the command line, you could also give a name of a file containing valid JSON to read them from.
+
+ le.pl ... --complete-params complete.json
+ 
+Crypt::LE::Challenge:: and Crypt::LE::Complete:: namespaces are suggested for new plugins.
+ 
 =head1 EXPORT
 
 Crypt::LE does not export anything by default, but allows you to import the following constants:
@@ -117,6 +129,7 @@ my $pkcs10_available = 0;
 # Crypt::OpenSSL::PKCS10 is offered as 'required' rather than 'recommended'. Depending on feedback it might
 # be moved to 'recommended' later, but the code will perform fine even without it. You will get a warning if
 # you try to generate a CSR and that module is not available.
+
 eval "use Crypt::OpenSSL::PKCS10;";
 unless ($@) {
     $pkcs10_available = 1;
@@ -160,7 +173,7 @@ my @primes = map { Crypt::OpenSSL::Bignum->new_from_decimal($_) } (
     719, 727, 733, 739, 743, 751
 );
 
-=head1 METHODS (API SETUP)
+=head1 METHODS (API Setup)
 
 The following methods are provided for the API setup. Please note that account key setup by default requests the resource directory from Let's Encrypt servers.
 This can be changed by resetting the 'autodir' parameter of the constructor.
@@ -201,7 +214,7 @@ Non-integer values are supported (so for example you can set it to 1.5 if you li
 
 =back
 
-B<Returns>: L<Crypt::LE> object.
+Returns: L<Crypt::LE> object.
 
 =cut
 
@@ -234,7 +247,7 @@ sub new {
 
 Loads the private account key from the file in PEM or DER formats.
 
-B<Returns>: OK | READ_ERROR | LOAD_ERROR | INVALID_DATA.
+Returns: OK | READ_ERROR | LOAD_ERROR | INVALID_DATA.
 
 =cut
 
@@ -256,7 +269,7 @@ sub load_account_key {
 
 Generates a new private account key of the $keysize bits (4096 by default). The key is additionally validated for not being divisible by small primes.
 
-B<Returns>: OK | INVALID_DATA.
+Returns: OK | INVALID_DATA.
 
 =cut
 
@@ -269,7 +282,7 @@ sub generate_account_key {
 
 =head2 account_key()
 
-B<Returns>: A previously loaded or generated private key in PEM format or undef.
+Returns: A previously loaded or generated private key in PEM format or undef.
 
 =cut
 
@@ -283,7 +296,7 @@ sub account_key {
 Loads Certificate Signing Requests from file. Domains list can be omitted or it can be given as a string of comma-separated names or as an array reference.
 If omitted, then names will be loaded from the CSR. If it is given, then the list of names will be verified against those found on CSR.
 
-B<Returns>: OK | READ_ERROR | LOAD_ERROR | INVALID_DATA | DATA_MISMATCH.
+Returns: OK | READ_ERROR | LOAD_ERROR | INVALID_DATA | DATA_MISMATCH.
 
 =cut
 
@@ -321,9 +334,8 @@ sub load_csr {
 
 Generates a new Certificate Signing Requests based on a new RSA key of $keysize bits (4096 by default). 
 Domains list is mandatory and can be given as a string of comma-separated names or as an array reference.
-I<Please note that you need Crypt::OpenSSL::PKCS10 module installed for this to work.>
 
-B<Returns>: OK | ERROR | INVALID_DATA.
+Returns: OK | ERROR | INVALID_DATA.
 
 =cut
 
@@ -351,7 +363,7 @@ sub generate_csr {
 
 =head2 csr()
 
-B<Returns>: A previously loaded or generated CSR in PEM format or undef.
+Returns: A previously loaded or generated CSR in PEM format or undef.
 
 =cut
 
@@ -361,7 +373,7 @@ sub csr {
 
 =head2 csr_key()
 
-B<Returns>: A private key of a previously generated CSR in PEM format or undef.
+Returns: A private key of a previously generated CSR in PEM format or undef.
 
 =cut
 
@@ -373,7 +385,7 @@ sub csr_key {
 
 Sets (or resets if no parameter is given) an email address that will be used for registration requests.
 
-B<Returns>: OK | INVALID_DATA.
+Returns: OK | INVALID_DATA.
 
 =cut
 
@@ -448,7 +460,7 @@ sub _set_csr {
 # API Workflow functions
 #====================================================================================================
 
-=head1 METHODS (API WORKFLOW)
+=head1 METHODS (API Workflow)
 
 The following methods are provided for the API workflow processing. All but C<accept_challenge()> methods interact with Let's Encrypt servers.
 
@@ -458,7 +470,7 @@ Loads resource pointers from Let's Encrypt. This method needs to be called befor
 will be called automatically upon account key loading/generation unless you have reset the 'autodir'
 parameter when creating a new Crypt::LE instance.
 
-B<Returns>: OK | LOAD_ERROR.
+Returns: OK | LOAD_ERROR.
 
 =cut
 
@@ -480,7 +492,7 @@ sub directory {
 
 Registers an account key with Let's Encrypt. If the key is already registered, it will be handled automatically.
 
-B<Returns>: OK | ERROR.
+Returns: OK | ERROR.
 
 =cut
 
@@ -520,7 +532,7 @@ sub register {
 
 Accepts Terms of Service set by Let's Encrypt.
 
-B<Returns>: OK | ERROR.
+Returns: OK | ERROR.
 
 =cut
 
@@ -536,7 +548,7 @@ sub accept_tos {
 Requests challenges for domains on your CSR. On error you can call failed_domains() method, which returns an array reference to domain names for which
 the challenge was not requested successfully.
 
-B<Returns>: OK | ERROR.
+Returns: OK | ERROR.
 
 =cut
 
@@ -621,40 +633,9 @@ B<handle_challenge_dns()>
 
 =back
 
-For example, you can create a module like this to handle the challenge:
-
- package Crypt::LE::Challenge::Simple;
+You can use L<Crypt::LE::Challenge::Simple> example module as a template.
  
- sub new { bless {}, shift }
- 
- sub handle_challenge_http {
-  my $self = shift;
-  my ($challenge, $params) = @_;
-  print "Domain: $challenge->{domain} expects '$challenge->{token}.$challenge->{fingerprint}' text in '$challenge->{token}' file\n";
-  <STDIN>;
-  return 1;
- };
- 
- 1;
-
-You can then use it as shown below from your program:
-
- use Crypt::LE;
- use Crypt::LE::Challenge::Simple;
- 
- my $le = Crypt::LE->new();
- my $simple_challenge = Crypt::LE::Challenge::Simple->new();
- ...
- $le->accept_challenge($simple_challenge);
- 
-Or you can use it with the provided client (le.pl), by adding a parameter to its command line:
-
- --handle-with Crypt::LE::Challenge::Simple
- 
-I<If you create a plugin handling challenges for Crypt::LE, please use Crypt::LE::Challenge:: namespace, so it would be easier to
-find those.> For a boilerplate of the plugin handling the challenges, see L<Crypt::LE::Challenge::Simple>.
- 
-B<Returns>: OK | ERROR | INVALID_DATA.
+Returns: OK | ERROR | INVALID_DATA.
 
 =cut
 
@@ -705,7 +686,7 @@ sub accept_challenge {
 Asks Let's Encrypt server to verify the results of the challenge. On error you can call failed_domains() method, which returns an array reference to domain names 
 for which the challenge was not verified successfully.
 
-B<Returns>: OK | ERROR.
+Returns: OK | ERROR.
 
 =cut
 
@@ -765,7 +746,7 @@ sub verify_challenge {
 
 Requests the certificate for your CSR.
 
-B<Returns>: OK | ERROR.
+Returns: OK | ERROR.
 
 =cut
 
@@ -787,7 +768,7 @@ sub request_certificate {
 
 Requests the issuer's certificate.
 
-B<Returns>: OK | ERROR.
+Returns: OK | ERROR.
 
 =cut
 
@@ -806,7 +787,7 @@ sub request_issuer_certificate {
 
 Revokes a certificate.
 
-B<Returns>: OK | READ_ERROR | ALREADY_DONE | ERROR.
+Returns: OK | READ_ERROR | ALREADY_DONE | ERROR.
 
 =cut
 
@@ -828,14 +809,14 @@ sub revoke_certificate {
 # API Workflow helpers
 #====================================================================================================
 
-=head1 METHODS (OTHER)
+=head1 METHODS (Other)
 
 The following methods are the common getters you can use to get more details about the outcome of the workflow run and return some retrieved data, such as
 registration info and certificates for your domains.
 
 =head2 tos()
 
-B<Returns>: The link to a Terms of Service document or undef.
+Returns: The link to a Terms of Service document or undef.
 
 =cut
 
@@ -846,7 +827,7 @@ sub tos {
 
 =head2 tos_changed()
 
-B<Returns>: True if Terms of Service have been changed (or you haven't yet accepted them). Otherwise returns false.
+Returns: True if Terms of Service have been changed (or you haven't yet accepted them). Otherwise returns false.
 
 =cut
 
@@ -856,7 +837,7 @@ sub tos_changed {
 
 =head2 new_registration()
 
-B<Returns>: True if new key has been registered. Otherwise returns false.
+Returns: True if new key has been registered. Otherwise returns false.
 
 =cut
 
@@ -866,7 +847,7 @@ sub new_registration {
 
 =head2 registration_info()
 
-B<Returns>: Registration information structure returned by Let's Encrypt for your key or undef.
+Returns: Registration information structure returned by Let's Encrypt for your key or undef.
 
 =cut
 
@@ -876,7 +857,7 @@ sub registration_info {
 
 =head2 certificate()
 
-B<Returns>: The last received certificate or undef.
+Returns: The last received certificate or undef.
 
 =cut
 
@@ -886,7 +867,7 @@ sub certificate {
 
 =head2 certificate_url()
 
-B<Returns>: The URL of the last received certificate or undef.
+Returns: The URL of the last received certificate or undef.
 
 =cut
 
@@ -896,7 +877,7 @@ sub certificate_url {
 
 =head2 issuer()
 
-B<Returns>: The issuer's certificate or undef.
+Returns: The issuer's certificate or undef.
 
 =cut
 
@@ -906,7 +887,7 @@ sub issuer {
 
 =head2 issuer_url()
 
-B<Returns>: The URL of the issuer's certificate or undef.
+Returns: The URL of the issuer's certificate or undef.
 
 =cut
 
@@ -916,7 +897,7 @@ sub issuer_url {
 
 =head2 domains()
 
-B<Returns>: An array reference to the domain names loaded from CSR or undef.
+Returns: An array reference to the domain names loaded from CSR or undef.
 
 =cut
 
@@ -927,7 +908,7 @@ sub domains {
 
 =head2 failed_domains()
 
-B<Returns>: An array reference to the domain names for which challenge processing has failed (on any of request/accept/verify steps) or undef.
+Returns: An array reference to the domain names for which challenge processing has failed (on any of request/accept/verify steps) or undef.
 
 =cut
 
@@ -937,7 +918,7 @@ sub failed_domains {
 
 =head2 error()
 
-B<Returns>: Last error (can be a code or a structure) or undef.
+Returns: Last error (can be a code or a structure) or undef.
 
 =cut
 
@@ -947,7 +928,7 @@ sub error {
 
 =head2 error_details()
 
-B<Returns>: Last error details if available or a generic 'error' string otherwise. Empty string if the last called method returned OK.
+Returns: Last error details if available or a generic 'error' string otherwise. Empty string if the last called method returned OK.
 
 =cut
 
@@ -1082,8 +1063,8 @@ L<https://do-know.com/>
 =head1 BUGS
 
 Considering that this module has been written in a rather quick manner after I decided to give a go to Let's Encrypt certificates
-and found that CPAN seems to be lacking some easy ways to leverage LE API from Perl, expect some bugs. The initial goal was to make
-this work, make it easy to use and possibly remove the need to use openssl command line.
+and found that CPAN seems to be lacking some easy ways to leverage LE API from Perl, expect some (hopefully minor) bugs. 
+The initial goal was to make this work, make it easy to use and possibly remove the need to use openssl command line.
 
 Please report any bugs or feature requests to C<bug-crypt-le at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Crypt-LE>.  I will be notified, and then you'll
