@@ -1,8 +1,9 @@
 package Crypt::LE::Challenge::Simple;
+use Data::Dumper;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 NAME
 
@@ -35,6 +36,9 @@ sub new { bless {}, shift }
 sub handle_challenge_http {
     my $self = shift;
     my ($challenge, $params) = @_;
+    if ($params) {
+        print "Parameters passed: " . Dumper($params);
+    }
     print "Domain: $challenge->{domain} expects '$challenge->{token}.$challenge->{fingerprint}' text in '$challenge->{token}' file\n";
     print "Press <Enter>";
     <STDIN>;
