@@ -9,7 +9,7 @@ use Crypt::LE ':errors';
 sub info { print shift, "\n" }
 sub bail { &info; exit 255 }
 
-my $VERSION = '0.07';
+my $VERSION = '0.08';
 
 my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, 
     $handle_with, $handle_as, $handle_params, $complete_with, $complete_params, $revoke, $verified, $debug, $help);
@@ -35,13 +35,24 @@ b) To use external modules to handle challenges and process completion while get
    le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing \\
          --handle-with Crypt::LE::Challenge::Simple --complete-with Crypt::LE::Complete::Simple
 
-   - See provided Crypt::LE::Challenge::Simple for an example of challenge handling module.
-   - See provided Crypt::LE::Complete::Simple for an example of completion handling module.
+   - See provided Crypt::LE::Challenge::Simple for an example of a challenge-handling module.
+   - See provided Crypt::LE::Complete::Simple for an example of a completion-handling module.
 
-c) To revoke a certificate:
+c) To pass parameters to external modules as JSON either directly or by specifying a file name:
+
+   le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing \\
+         --handle-with Crypt::LE::Challenge::Simple --complete-with Crypt::LE::Complete::Simple \\
+         --handle-params '{"key1": 1, "key2": 2, "key3": "something"}' --complete-params complete.json
+
+d) To just generate the keys and CSR:
+
+   le.pl  --key account.key --csr domain.csr --csr-key domain.key --domains "www.domain.ext,domain.ext" --generate-missing --generate-only
+
+e) To revoke a certificate:
 
    le.pl --key account.key --crt domain.crt --revoke
 
+
 [!] NOTE: by default a staging server is used, which does not provide trusted certificates. This is to aviod hitting a 
           rate limits on Let's Encrypt live server. To generate an actual certificate, always add --live option.
 
@@ -79,7 +90,7 @@ unless ($account_key and (-r $account_key or $generate_missing)) {
     bail $usage;
 }
 
-unless ($crt) {
+unless ($crt or $generate_only) {
     bail "Please specify a file name for the certificate.";
 }
 
@@ -196,7 +207,7 @@ if ($complete_handler) {
     }
 }
 
-info "The job is done, enjoy your certificate! For feedback and bug reports contact me at [ https://do-know.com ]";
+info "\nThe job is done, enjoy your certificate! For feedback and bug reports contact me at [ https://do-know.com ]\n";
 
 sub _load_params {
     my ($src, $type) = @_;
